package com.tencent.edu.kernel.http

import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import com.google.gson.JsonObject
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.kernel.http.data.HttpResponse
import com.tencent.edu.kernel.http.retrofit.RetrofitManager
import com.tencent.edu.module.audiovideo.entity.ContractTeacherInfo
import com.tencent.edu.module.homepage.model.EnjoyStudyCardDetail
import com.tencent.edu.module.setting.DebugDcLogActivity
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.Scheduler
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers

/**
 * @author aidanyang
 * @date 2020-08-11
 */
object HttpModel {

    private val service = RetrofitManager.create(HttpApi::class.java)

    private val disposableMap = hashMapOf<Any, ArrayList<Disposable>>()

    /**
     * 获取课件更新信息
     */
    @JvmStatic
    @JvmOverloads
    fun fetchCoursewareNotice(cid: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchCoursewareNotice(cid), observer, tag)
    }

    /**
     * 获取课件下载url
     */
    @JvmStatic
    @JvmOverloads
    fun fetchDownloadUrl(id: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchDownloadUrl(id), observer, tag)
    }

    /**
     * 获取直播间联系信息
     */
    @JvmStatic
    @JvmOverloads
    fun fetchContractTeacherInfo(courseId: String, observer: Observer<HttpResponse<ContractTeacherInfo>>, tag: Any? = null) {
        setSubscribe(service.fetchContractTeacherInfo(courseId), observer, tag)
    }

    /**
     * 设置直播间联系弹窗不再自动弹出
     */
    @JvmStatic
    @JvmOverloads
    fun setNoMoreNotify(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.setNoMoreNotify(termId), observer, tag)
    }

    /**
     * 上传全链路日志
     */
    @JvmStatic
    @JvmOverloads
    fun reportDcLog(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        val host = if (DebugDcLogActivity.isTestEnv()) {
            "http://debug.edu.oa.com"
        } else {
            "https://report.edu.qq.com"
        }
        setSubscribe(service.reportDcLog(host, req), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun checkUserSetPhone(courseId: String, termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkUserSetPhone(
                MiscUtils.getBkn(),
                courseId,
                termId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCollectStatus(contentId: String, externalFlag: Int, seriesId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCollectStatus(
                contentId,
                externalFlag,
                seriesId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCollectStatus(svid: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCollectStatus(svid), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun updateCollectStatus(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.updateCollectStatus(req), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getContentInfo(preview: Int, contentId: String, externalFlag: Int, seriesId: String?, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getContentInfo(
                preview,
                contentId,
                externalFlag,
                seriesId,
                1
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getContentRecommend(page: Int, count: Int, platform: Int, mt: String, st: String, tt: String, label: String, sceneId: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getContentRecommend(
                page,
                count,
                platform,
                mt,
                st,
                tt,
                label,
                sceneId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun fetchPushMsgWithReqType(url: String, req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchPushMsgWithReqType(
                url,
                req
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun fetchExplainList(termId: String, pageSize: Int, lastMsgId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchExplainList(
                termId,
                pageSize,
                lastMsgId,
                1
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun checkWithinWhiteList(q: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkWithinWhiteList(q), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun requestBottomTabIcon(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestBottomTabIcon(), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun flutterGet(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.flutterGet(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getAppSplash(platform: Int, cookies: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getAppSplash(platform, cookies), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun checkSmartCampus(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkSmartCampus(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestSmartCampusLoginKey(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestSmartCampusLoginKey(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getCourses(present_flag: Int, is_full_info: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCourses(present_flag, is_full_info), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getAdBindPhoneChannel(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getAdBindPhoneChannel(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getTermCanApply(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getTermCanApply(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestOperation(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestOperation(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun grantOrGetStudyCard(enterPageIndex: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.grantOrGetStudyCard(enterPageIndex), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCouponMarketing(url:String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCouponMarketing(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getStudyCourseDetail(termId: String, observer: Observer<HttpResponse<EnjoyStudyCardDetail>>, tag: Any? = null) {
        setSubscribe(service.getStudyCourseDetail(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun syncCanGrantStudyCard(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.syncCanGrantStudyCard(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestSSOAuth(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestSSOAuth(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun request(url: String, needCookie:Boolean = true, observer: Observer<JsonObject>, tag: Any? = null) {
        if(needCookie){
            setSubscribe(service.requestWithCookie(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        }else{
            setSubscribe(service.request(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        }
    }

    /**
     * 监听生命周期，在onDestroy时清空tag为当前owner的所有请求
     */
    @JvmStatic
    fun addLifecycleObserver(owner: LifecycleOwner) {
        owner.lifecycle.addObserver(object : DefaultLifecycleObserver {
            override fun onDestroy(owner: LifecycleOwner) {
                cancel(owner)
            }
        })
    }

    /**
     * 通过tag取消一个请求
     */
    @JvmStatic
    fun cancel(tag: Any) {
        disposableMap.remove(tag)?.run {
            for (disposable in this) {
                if (disposable.isDisposed) {
                    return@run
                }

                disposable.dispose()
            }
            clear()
        }
    }

    /**
     * 取消当前所有请求
     */
    @JvmStatic
    fun cancelAll() {
        while (disposableMap.isNotEmpty()) {
            cancel(disposableMap.keys.last())
        }
    }

    /**
     * 订阅网络请求
     */
    private fun <T> setSubscribe(
            observable: Observable<T>,
            // 请求结果Observer
            observer: Observer<T>,
            // 设置tag，可用于取消请求
            tag: Any? = null,
            // IO线程访问网络
            subscribeOn: Scheduler = Schedulers.io(),
            // 设置请求结果在子线程或主线程回调
            observeOn: Scheduler = Schedulers.newThread()
    ) {

        observable.subscribeOn(subscribeOn)
                .observeOn(observeOn)
                .subscribe(if (tag == null) observer else object : Observer<T> {
                    var disposable: Disposable? = null

                    override fun onSubscribe(d: Disposable) {
                        disposable = d
                        if (disposableMap[tag] == null) {
                            disposableMap[tag] = arrayListOf(d)
                        } else {
                            disposableMap[tag]?.add(d)
                        }
                    }

                    override fun onNext(t: T) {
                        observer.onNext(t)
                    }

                    override fun onError(e: Throwable) {
                        observer.onError(e)
                    }

                    override fun onComplete() {
                        observer.onComplete()
                        disposableMap[tag]?.remove(disposable)
                    }
                })
    }

}